unit UDrawMethod;

{$I beanz.inc}

interface

uses
  Classes, SysUtils, ExtCtrls,Graphics, Controls,
  LCLType, IntfGraphics, fpImage, GraphType, DateUtils,
  uCrossPlateForm, FastImage,
  {$IFDEF OPENGL}GL, GLExt, OpenGLContext,{$ENDIF}
  LclIntf;

type
  TPaintObject = (poImage, poFastImage, poPaintBox, poOpenGL);


  { TDrawMethod }

  TDrawMethod = class
  private
    FControl: TControl;
    FFPS: Single;
    FParent: TWinControl;
  public
    Caption: string;
    Description: TStringList;
    Terminated: Boolean;
    FrameWidth, FrameHeight : Integer;
    FrameDuration: Double;//TDateTime;

    PaintObject: TPaintObject;
    FrameCounter: Integer;
    FrameCounterStart: Int64;// TDateTime;
    FrameCounterStop: Int64;//TDateTime;
    Iterations : Integer;


    perfCounter, framePerf: Int64;

    function GetFPS: Single;
    property FPS: Single read FFPS write FFPS;
    procedure Init(Parent: TWinControl; AWidth, AHeight:Integer); virtual;
    procedure Done; virtual;
    constructor Create; virtual;
    destructor Destroy; override;
    procedure DrawFrame(); virtual;
    procedure DrawFrameTiming();

    procedure UpdateSettings; virtual;
    property Control: TControl read FControl;
  end;

  TDrawMethodClass = class of TDrawMethod;

  { TDrawMethodImage }

  TDrawMethodImage = class(TDrawMethod)
    Image: TImage;
    procedure UpdateSettings; override;
    procedure Init(Parent: TWinControl; AWidth, AHeight:Integer); override;
    procedure Done; override;
  end;

    { TDrawMethodImage }

  TDrawMethodFastImage = class(TDrawMethod)
    Image: TFastImage;
    procedure UpdateSettings; override;
    procedure Init(Parent: TWinControl; AWidth, AHeight:Integer); override;
    procedure Done; override;
  end;

  { TDrawMethodPaintBox }

  TDrawMethodPaintBox = class(TDrawMethod)
    PaintBox: TPaintBox;
    procedure Paint(Sender: TObject); virtual;
    procedure UpdateSettings; override;
    procedure Init(Parent: TWinControl; AWidth, AHeight:Integer); override;
    procedure Done; override;
  end;

  {$IFDEF OPENGL}

  { TDrawMethodOpenGL }

  TDrawMethodOpenGL = class(TDrawMethod)
    OpenGLControl: TOpenGLControl;
    TextureId: GLuint;
    OpenGLBitmap: Pointer;
    procedure UpdateSettings; override;
    procedure InitGL;
    procedure OpenGLControlResize(Sender: TObject);
    procedure Init(AParent: TWinControl; Size: TPoint; PixelFormat: TPixelFormat); override;
    procedure Done; override;
  end;

  {$ENDIF}


implementation


{ TDrawMethodPaintBox }

procedure TDrawMethodPaintBox.Paint(Sender: TObject);
begin

end;

procedure TDrawMethodPaintBox.UpdateSettings;
begin
  inherited UpdateSettings;
  PaintBox.ControlStyle := FParent.ControlStyle;
end;

procedure TDrawMethodPaintBox.Init(Parent: TWinControl; AWidth, AHeight:Integer);
begin
  inherited;
  PaintBox := TPaintBox.Create(Parent);
  PaintBox.Parent := Parent;
  PaintBox.SetBounds(0, 0, AWidth, AHeight);
  PaintBox.OnPaint := @Paint;
  PaintBox.Show;
  UpdateSettings;
end;

procedure TDrawMethodPaintBox.Done;
begin
  FreeAndNil(PaintBox);
  inherited Done;
end;

{ TDrawMethodImage }

procedure TDrawMethodImage.UpdateSettings;
begin
  inherited;
  Image.ControlStyle := FParent.ControlStyle;
end;

procedure TDrawMethodImage.Init(Parent: TWinControl; AWidth, AHeight:Integer);
begin
  inherited;
  Image := TImage.Create(Parent);
  Image.Parent := Parent;
  Image.SetBounds(0, 0, AWidth, AHeight);
  Image.Picture.Bitmap.PixelFormat := pf32bit;
  Image.Picture.Bitmap.SetSize(AWidth, AHeight);
  Image.Show;
  UpdateSettings;
end;

procedure TDrawMethodImage.Done;
begin
  FreeAndNil(Image);
  inherited Done;
end;


{ TDrawMethodFastImage }

procedure TDrawMethodFastImage.UpdateSettings;
begin
  inherited;
  Image.ControlStyle := FParent.ControlStyle;
end;

procedure TDrawMethodFastImage.Init(Parent: TWinControl; AWidth, AHeight:Integer);
begin
  inherited;
  Image := TFastImage.Create(Parent);
  Image.Parent := Parent;
  Image.SetBounds(0, 0, AWidth, AHeight);
  Image.Picture.SetSize(AWidth, AHeight);
  //Image.Show;
  UpdateSettings;
end;

procedure TDrawMethodFastImage.Done;
begin
  FreeAndNil(Image);
  inherited Done;
end;


{$IFDEF OPENGL}

{ TDrawMethodOpenGL }

procedure TDrawMethodOpenGL.Init(AParent: TWinControl; Size: TPoint; PixelFormat: TPixelFormat);
begin
  inherited;
  OpenGLControl := TOpenGLControl.Create(AParent);
  with OpenGLControl do begin
    Name := 'OpenGLControl';
    Parent := AParent;
    SetBounds(0, 0, Size.X, Size.Y);
    InitGL;
    //OnPaint := OpenGLControl1Paint;
    OnResize := OpenGLControlResize;
  end;
  GetMem(OpenGLBitmap, OpenGLControl.Width * OpenGLControl.Height * SizeOf(Integer));
  UpdateSettings;
end;

procedure TDrawMethodOpenGL.Done;
begin
  FreeMem(OpenGLBitmap, OpenGLControl.Width * OpenGLControl.Height);
  FreeAndNil(OpenGLControl);
  inherited;
end;

procedure TDrawMethodOpenGL.OpenGLControlResize(Sender: TObject);
begin
  glViewport(0, 0, OpenGLControl.Width, OpenGLControl.Height);
end;

procedure TDrawMethodOpenGL.UpdateSettings;
begin
  inherited UpdateSettings;
  OpenGLControl.ControlStyle := FParent.ControlStyle;
end;

procedure TDrawMethodOpenGL.InitGL;
begin
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  glOrtho(0, OpenGLControl.Width, OpenGLControl.Height, 0, 0, 1);
//  glOrtho(0, 1, 1, 0, 0, 1);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glDisable(GL_DEPTH_TEST);
  glViewport(0, 0, OpenGLControl.Width, OpenGLControl.Height);
  //gluPerspective( 45.0, (GLfloat)(OpenGLControl1.Width)/(GLfloat)(OpenGLControl1.Height), 0.1f, 500.0 );

    //glFrustum (-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
    //glTranslatef (0.0, 0.0,-3.0);
  //  glClearColor(0.0, 0.0, 0.0, 1.0);

  glGenTextures(1, @TextureId);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
end;

{$ENDIF}


{ TDrawMethod }

function TDrawMethod.GetFPS: Single;
var
  //StopTime: TDateTime;
  StopTime:Int64;
begin

  if FrameCounter > 0 then
  begin
    if FrameCounterStop <> 0 then
      StopTime := FrameCounterStop
    else StopTime := StartPrecisionTimer; //NowPrecise;

    Dec(StopTime, FrameCounterStart);
    FFPS := (FrameCounter * PerformanceFrequency) / StopTime
  end
  else result :=FFPS;
end;

procedure TDrawMethod.Init(Parent: TWinControl; AWidth, AHeight:Integer);
begin
  FParent := Parent;
  FrameWidth:=AWidth;
  FrameHeight:=AHeight;
end;

procedure TDrawMethod.Done;
begin

end;

constructor TDrawMethod.Create;
begin
  Description := TStringList.Create;
end;

destructor TDrawMethod.Destroy;
begin
  FreeAndNil(Description);
  inherited;
end;

procedure TDrawMethod.DrawFrame();
begin

end;

procedure TDrawMethod.DrawFrameTiming();
var
  StartTime: Int64;//TDateTime;
begin
  StartTime :=StartPrecisionTimer;// NowPrecise;
  DrawFrame();
  FrameDuration :=StopPrecisionTimer(StartTime);
end;



procedure TDrawMethod.UpdateSettings;
begin
end;

end.

