unit umainform;


{$I beanz.inc}

interface

uses
  SysUtils, Forms, Controls, ActnList,
  Menus, ExtCtrls, StdCtrls, Spin, ComCtrls,
  fpImage, Contnrs, Dialogs,
  UDrawMethod, uCrossPlateForm, Classes;


type
  { TMainForm }
  TMainForm = class(TForm)
    actDisplayInfos: TAction;
    actExit: TAction;
    actStartOneTest: TAction;
    actStopTest: TAction;
    actStartAllTest: TAction;
    actViewTestForm: TAction;
    ActionList1: TActionList;
    btnStartAllTest: TButton;
    btnStartOneTest: TButton;
    btnStopTest: TButton;
    chkDoubleBuffer: TCheckBox;
    chkEraseBackground: TCheckBox;
    chkFullScreen: TCheckBox;
    chkOpaque: TCheckBox;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    lblCurrentMethod: TLabel;
    lblMethodProgress: TLabel;
    LVMethods: TListView;
    MainMenu1: TMainMenu;
    MainMenu2: TMainMenu;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    mmoDescription: TMemo;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    muiSysInfo: TMenuItem;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    SpinEditHeight: TSpinEdit;
    SpinEditIterations: TSpinEdit;
    SpinEditWidth: TSpinEdit;
    TimerUpdateList: TTimer;
    TimerUpdateSettings: TTimer;
    procedure actDisplayInfosExecute(Sender: TObject);
    procedure actExitExecute(Sender: TObject);
    procedure actStartAllTestExecute(Sender: TObject);
    procedure actStartOneTestExecute(Sender: TObject);
    procedure actStopTestExecute(Sender: TObject);
    procedure actViewTestFormExecute(Sender: TObject);
    procedure chkDoubleBufferChange(Sender: TObject);
    procedure chkEraseBackgroundChange(Sender: TObject);
    procedure chkFullScreenChange(Sender: TObject);
    procedure chkOpaqueChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure LVMethodsData(Sender: TObject; Item: TListItem);
    procedure LVMethodsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure SpinEditHeightChange(Sender: TObject);
    procedure SpinEditIterationsChange(Sender: TObject);
    procedure SpinEditWidthChange(Sender: TObject);
    procedure TimerUpdateListTimer(Sender: TObject);
    procedure TimerUpdateSettingsTimer(Sender: TObject);
  private
    { private declarations }
    FCurrentMethod: TDrawMethod;
    MethodIndex: Integer;
    TestTerminated: Boolean;
    TestTimeout: Real;
    SceneFrameCount:Integer;
    DrawMethodClasses: array of TDrawMethodClass;

    procedure TestMethod(Method: TDrawMethod);

    procedure UpdateMethodList;
    procedure UpdateInterface;
    procedure RegisterDrawMethods;
    procedure RegisterDrawMethod(MethodClass: TDrawMethodClass);
  public
    { public declarations }
    SingleTestActive,TestActive:boolean;
    FrameWidth: Integer;
    FrameHeight: Integer;
    DrawMethods: TObjectList; // TObjectList<TDrawMethod>

    property CurrentMethod: TDrawMethod read FCurrentMethod write FCurrentMethod;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

uses
  uInfosForm,
  UDrawForm,
  // TFastBitmap + TFastImage
  //uFastimagemethod_clear,
  uFastImagePixelMethod,
  uFastImageColorsMethod,
  // TFastBitmap + TLCLFastBitmapHandler
  ufastBitmapHandlerPixelsMethod,
  // TImage + TBitmap.Canvas
  UCanvasPixelsMethod,
  UCanvasPixelsUpdateLockMethod,
  // TImage + TBitmap.RawImage.Data
  UBitmapRawImageDataPixelsPtrMethod,
  // TPaintBox + TBitmap.RawImage.Data
  UBitmapRawImageDataPaintBoxMethod,
  // TImage + TLazIntfImage
  ULazIntfImageColorsMethod,
  // Dummy
  uDummyMethod;
{ TMainForm }

procedure TMainForm.actDisplayInfosExecute(Sender: TObject);
begin
  SystemInfosForm.Show;
end;

procedure TMainForm.actExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.actViewTestFormExecute(Sender: TObject);
begin
  DrawForm.Show;
end;

procedure TMainForm.chkDoubleBufferChange(Sender: TObject);
begin
  DrawForm.DoubleBuffered := chkDoubleBuffer.Checked;
end;

procedure TMainForm.chkEraseBackgroundChange(Sender: TObject);
begin
  DrawForm.EraseBackgroundEnabled := chkEraseBackground.Checked;
end;

procedure TMainForm.chkFullScreenChange(Sender: TObject);
begin
  DrawForm.FullScreen:=chkFullScreen.Checked;
end;

procedure TMainForm.chkOpaqueChange(Sender: TObject);
begin
  if chkOpaque.Checked then
    DrawForm.ControlStyle := DrawForm.ControlStyle + [csOpaque]
  else
    DrawForm.ControlStyle := DrawForm.ControlStyle - [csOpaque];

  if Assigned(FCurrentMethod) then FCurrentMethod.UpdateSettings;
end;

procedure TMainForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  actStopTest.Execute;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  NewDrawMethod: TDrawMethod;
  I: Integer;

begin
  Randomize;

  FrameWidth:=320;
  FrameHeight:=240;

  TestActive:=false;
  SingleTestActive:=false;
  RegisterDrawMethods;
  DrawMethods := TObjectList.Create;
  for I := 0 to High(DrawMethodClasses) do begin
    NewDrawMethod := DrawMethodClasses[I].Create;
    DrawMethods.Add(NewDrawMethod);
  end;
  UpdateMethodList;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  LVMethods.Clear;
  FreeAndNil(DrawMethods);
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  DrawForm.Show;
  UpdateMethodList;
  UpdateInterface;
  DrawForm.Left := Left + Width;
  DrawForm.Top := Top;
end;

procedure TMainForm.LVMethodsData(Sender: TObject; Item: TListItem);
begin
  if (Item.Index >= 0) and (Item.Index < DrawMethods.Count) then
  with TDrawMethod(DrawMethods[Item.Index]) do
  begin
    Item.Caption := Caption;
    FPS := GetFPS;
    Item.SubItems.Add( Format('%.*f FPS', [3, FPS]));

    if FrameDuration > 0 then
      Item.SubItems.Add(Format('%.*fms', [6,FrameDuration]))
    else Item.SubItems.Add('0');
  end;
end;

procedure TMainForm.LVMethodsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin

  FCurrentMethod:=TDrawMethod(DrawMethods[LVMethods.Selected.Index]);
  UpdateInterface;
  lblCurrentMethod.Caption:=TDrawMethod(DrawMethods[LVMethods.Selected.Index]).Caption;
  SpinEditIterations.Value := TDrawMethod(DrawMethods[LVMethods.Selected.Index]).Iterations;
  MmoDescription.Lines.Assign(TDrawMethod(DrawMethods[LVMethods.Selected.Index]).Description);
end;

procedure TMainForm.SpinEditHeightChange(Sender: TObject);
begin
  FrameHeight := SpinEditHeight.Value;
end;

procedure TMainForm.SpinEditIterationsChange(Sender: TObject);
begin
  if Assigned(FCurrentMethod) then
  begin
     FCurrentMethod.Iterations := SpinEditIterations.Value;
  end;
end;

procedure TMainForm.SpinEditWidthChange(Sender: TObject);
begin
  FrameWidth := SpinEditWidth.Value;
end;

procedure TMainForm.TimerUpdateListTimer(Sender: TObject);
begin
  UpdateMethodList;
end;

procedure TMainForm.TimerUpdateSettingsTimer(Sender: TObject);
begin
  if (FrameWidth <> DrawForm.FrameWidth) or (FrameHeight <> DrawForm.FrameHeight) then
  begin
    DrawForm.FrameWidth := FrameWidth;
    DrawForm.FrameHeight := FrameHeight;
    DrawForm.ClientWidth := FrameWidth;
    DrawForm.ClientHeight := FrameHeight;
  end;
end;

procedure TMainForm.TestMethod(Method: TDrawMethod);
//var
//  StartTime: TDateTime;
begin
  TestTerminated:=false;
  FCurrentMethod := Method;
  UpdateInterface;
  FCurrentMethod.Init(DrawForm, FrameWidth, FrameHeight);
  with FCurrentMethod do
  begin
    SceneFrameCount := Iterations;
    //StartTime := NowPrecise;
    FrameCounterStart := StartPrecisionTimer;//NowPrecise;
    FrameCounterStop := 0;
    FrameCounter := 0;
    repeat
      lblMethodProgress.Caption := 'Iteration en cours : '+inttostr(FrameCounter)+'/'+inttostr(SceneFrameCount);
      DrawFrameTiming();
      Application.ProcessMessages;
      Inc(FrameCounter);
    until TestTerminated  or (FrameCounter > SceneFrameCount);
         // or ((TestTimeout > 0) and ((NowPrecise - StartTime) > OneSecond * TestTimeout))

    FrameCounterStop := StartPrecisionTimer; //NowPrecise;
    lblMethodProgress.Caption := '';
    mmoDescription.Lines.Clear;
    lblCurrentMethod.Caption:='';
    Done;
  end;
end;

procedure TMainForm.UpdateMethodList;
begin
  LVMethods.Items.Count := DrawMethods.Count;
  LVMethods.Refresh;
end;

procedure TMainForm.UpdateInterface;
begin
  btnStartAllTest.Enabled := (not TestActive) and (not SingleTestActive);
  btnStartOneTest.Enabled := (not TestActive) and (not SingleTestActive) and Assigned(LVMethods.Selected);;
  btnStopTest.Enabled := TestActive or SingleTestActive;
  SpinEditWidth.MaxValue := Screen.DesktopWidth;
  SpinEditHeight.MaxValue := Screen.DesktopHeight;
  ChkDoubleBuffer.Checked := DrawForm.DoubleBuffered;
  ChkEraseBackground.Checked := DrawForm.EraseBackgroundEnabled;
  ChkFullScreen.Checked := DrawForm.FullScreen;
  ChkOpaque.Checked := csOpaque in DrawForm.ControlStyle;
  if (Assigned(FCurrentMethod)) then
  begin
    spinEditIterations.Enabled:=true;
    spinEditIterations.Value:=CurrentMethod.Iterations;
  end
  else spinEditIterations.Enabled:=false;
end;

procedure TMainForm.RegisterDrawMethods;
begin
   // Dummy Methode
   RegisterDrawMethod(TDummyMethod);
   // Methode Clear - Effacement du bitmap avec une couleur donnée
   //RegisterDrawMethod(TFastimagemethodClear);

   // Methode Pixels - Remplissage du bitmap avec des couleurs aleatoires
   // TImage/TBitmap.Canvas
   RegisterDrawMethod(TCanvasPixelsMethod);
   RegisterDrawMethod(TCanvasPixelsUpdateLockMethod);
   // TImage/TBitmap.RawImage.Data
   RegisterDrawMethod(TBitmapRawImageDataPointerMethod);
   // TPaintBox/TBitmap.RawImage.Data
   RegisterDrawMethod(TBitmapRawImageDataPaintBoxMethod);
   // TImage/TLazIntfImage
   RegisterDrawMethod(TLazIntfImageColorsMethod);
   // TFastImage/TFastBitmap
   RegisterDrawMethod(TFastImagePixelMethod);
   RegisterDrawMethod(TFastImageColorsMethod);
   // TFastBitmap/TLCLFastBitmapHandler
   RegisterDrawMethod(TFastBitmapHandlerPixelsMethod);

end;

procedure TMainForm.RegisterDrawMethod(MethodClass: TDrawMethodClass);
begin
  SetLength(DrawMethodClasses, Length(DrawMethodClasses) + 1);
  DrawMethodClasses[High(DrawMethodClasses)] := MethodClass;
end;

procedure TMainForm.actStartAllTestExecute(Sender: TObject);
var
  I: Integer;
begin
  try
    TestActive:=True;
    UpdateInterface;
    TimerUpdateList.Enabled := True;
    TestTerminated := False;
    TestTimeout := 1.0;//FloatSpinEdit1.Value;

      for I := 0 to DrawMethods.Count - 1 do
      begin
        with TDrawMethod(DrawMethods[I]) do
        begin
          lblCurrentMethod.Caption:=TDrawMethod(DrawMethods[I]).Caption;
          MmoDescription.Lines.Assign(TDrawMethod(DrawMethods[I]).Description);
          SpinEditIterations.Value := TDrawMethod(DrawMethods[I]).Iterations;
          TestMethod(TDrawMethod(DrawMethods[I]));
          if TestTerminated then Break;
        end;
      end;

  finally
    TestActive:=false;
    UpdateInterface;
    TimerUpdateList.Enabled := False;
    UpdateMethodList
  end;

end;

procedure TMainForm.actStartOneTestExecute(Sender: TObject);
begin
  if Assigned(LVMethods.Selected) then
   try
     SingleTestActive := True;
     UpdateInterface;
     TimerUpdateList.Enabled := True;
     TestTerminated := False;
     MethodIndex := LVMethods.Selected.Index;
     TestTimeout := -1;
     if Assigned(CurrentMethod) then TestMethod(CurrentMethod);

   finally
     SingleTestActive := False;
     UpdateInterface;
     TimerUpdateList.Enabled := False;
     UpdateMethodList;
     FCurrentMethod:=nil;
   end;
end;

procedure TMainForm.actStopTestExecute(Sender: TObject);
begin
  TestTerminated := True;
  UpdateInterface;
end;

end.

