//------------------------------------------------------------------------------
// Defines.inc
//------------------------------------------------------------------------------
// Historique :
// 18/11/16 - BeanzMaster - Creation
//------------------------------------------------------------------------------
// Description :
// Definition des paramètres globaux de compilation
//------------------------------------------------------------------------------
{$DEFINE BEANZ_ENGINE}
//==============================================================================
//----[ Options de compilation ]------------------------------------------------
//==============================================================================
{$IFDEF FPC}
  {$mode objfpc}
  {$ASMMODE INTEL}

  {.$MODE FPC}
  {.$MODE DELPHI}
  {.$MODE MACPAS}
  {$LONGSTRINGS ON}
  {$H+,O+,X+}
{$ELSE}
  {$EXTENDEDSYNTAX ON}
  {$ALIGN ON}
  {$OPTIMIZATION ON}
{$ENDIF}
//------------------------------------------------------------------------------

//==============================================================================
//----[ Mode Debug ]------------------------------------------------------------
// Activer / désactiver le mode de débogage
//==============================================================================
{.$DEFINE DEBUG_ON}

{$IFDEF DEBUG_ON}
  // Activer tous les avertissements 'Unsafe Type' / 'Unsafe Code'
  {$DEFINE DEBUG_ENABLE_UNSAFE_WARNINGS}
  // Activer / désactiver le systeme de LOG
  {.$DEFINE LOGGING_ON}
  // Activer le mode de débogage OpenGL disponible uniquement avec OpenGL 4. 0 et supérieur
  {.$DEFINE GLS_OPENGL_DEBUG}
{$ENDIF}
//------------------------------------------------------------------------------

//==============================================================================
//----[ Détection des paramètres systeme ]--------------------------------------
// Détectection de la plateforme et du compilateur et paramétrage en fonction
//==============================================================================


//------------------------------------------------------------------------------
// Détection du compilateur
//------------------------------------------------------------------------------
// Free Pascal Compiler Lazarus / CodeTyphon
{$IFDEF FPC}
  {$IFNDEF WINDOWS}
    {$LINKLIB c}
  {$ENDIF}
{$ENDIF}

//------------------------------------------------------------------------------
// Delphi RAD Studio
{$IFNDEF FPC}
{$IFDEF VER310}   // Delphi Berlin
    {.$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER300}   // Delphi Seatlle
    {.$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER290}   // Delphi XE 8
    {.$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER280}   // Delphi XE 7
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER270}   // Delphi XE 6
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER260}   // Delphi XE 5
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER250}   // Delphi XE 4
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER240}   // Delphi XE 3
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER230}   // Delphi XE 2
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER220}   // Delphi XE 1
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER210}   // Delphi 2010
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER200}   // Delphi 2009
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER185}   // Delphi 2007/2006
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER180}   // Delphi 2006
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER170}   // Delphi 2005
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER220}   // Delphi XE 1
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER220}   // Delphi XE 1
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER150}   // Delphi 7
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER140}   // Delphi 6
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
{$IFDEF VER130}   // Delphi 5
    {$DEFINE UNSUPPORTED_DELPHI_VERSION}
{$ENDIF}
//----[ Support Additionnel ]---------------------------------------------------
//Activer la prise en charge des images png (en utilisant la bibliothèque pngimage.pas)
//GLS_PngImage_SUPPORT est automatiquement défini si Delphi> = 2009 (voir ci-dessous)
//Aussi, Définissez 'Auto' Redimensionner à la taille multiple la plus proche 8
//PNG est pris en charge par Delphi depuis Dephi 2009
{$DEFINE GLS_PngImage_SUPPORT}
{$DEFINE GLS_PngImage_RESIZENEAREST}

{$ENDIF}
//------------------------------------------------------------------------------
// Détection de l'OS
//------------------------------------------------------------------------------
{$IFDEF WINDOWS}
  // Defini le mode couleur BGRA pour l'affichage
  {$DEFINE BGRAIMAGE}
  {$IFDEF WIN32}
    // Désactiver les routines en assembleurs
    {.$DEFINE NO_ASM_OPTIMIZATIONS}
  {$ENDIF}
  {$IFDEF WIN64}
    // Désactiver les routines en assembleurs car non pris en charge sur des systemes 64bits
    {$DEFINE NO_ASM_OPTIMIZATIONS}
  {$ENDIF}
  // Par défaut, GLX n'est pas pris en charge sur Windows, bien que les implémentations Windows existent
  {$UNDEF GLX_SUPPORT}
  {$UNDEF X11_SUPPORT}
  {$DEFINE WGL_SUPPORT}
{$ENDIF}
//------------------------------------------------------------------------------
{$IFDEF LINUX}
  {$IFDEF LCLQT}
    // Mode d'affichage BGRA
    {$DEFINE BGRAIMAGE}
  {$ELSE}
    // Mode d'affichage RGBA
    {$DEFINE RGBAIMAGE}
  {$ENDIF}
  {$DEFINE NO_ASM_OPTIMIZATIONS}
  {$DEFINE UNIX}
  {$UNDEF WGL_SUPPORT}
  //Activer le support de X Window
  {$DEFINE X11_SUPPORT}
  {$DEFINE GLX_SUPPORT}
{$ENDIF}
//------------------------------------------------------------------------------
{$IFDEF DARWIN}
  {$IFDEF LCLQT}
    // Mode d'affichage BGRA
    {$DEFINE BGRAIMAGE}
  {$ELSE}
    // Mode d'affichage RGBA
    {$DEFINE RGBAIMAGE}
  {$ENDIF}
  {$DEFINE NO_ASM_OPTIMIZATIONS}
  {$UNDEF WGL_SUPPORT}
  {$UNDEF GLX_SUPPORT}
  {$UNDEF X11_SUPPORT}
{$ENDIF}
{$IFDEF UNSUPPORTED_DELPHI_VERSION}
  {$DEFINED UNSUPPORTED}
{$ENDIF}

//==============================================================================
//----[ Paramètres supplémentaires ]--------------------------------------------
//==============================================================================

// Activez la prise en charge de certaines optimisations en désactivant les
// vérifications qui se produisent très souvent.
// Avertissement: il désactive certaines options (comme ShowAxis et OptSaveGLStack)
{.$DEFINE OPTIMIZATIONS_ON}

// De-Activate assembly routines in units.
{.$DEFINE NO_ASM_OPTIMIZATIONS}

// De-Activate inline directive in  units.
{.$DEFINE NO_INLINE}

// De-Activate regions in units.
{.$DEFINE NO_REGIONS}


//==============================================================================

{$IFDEF UNSUPPORTED}
  Error: unsupported version, requires at least Delphi XE2
{$ENDIF}

{$IFNDEF NO_INLINE}
  {$DEFINE INLINE_ON}
{$ENDIF}

{$IFNDEF NO_REGIONS}
  {$DEFINE REGIONS_ON}
{$ENDIF}
//==============================================================================


