//==============================================================================
// CustomFastBitmapHandler.pas
//------------------------------------------------------------------------------
// Historique :
// 18/11/16 - BeanzMaster - Creation
//------------------------------------------------------------------------------
// Description :
// Composant graphique optimisé pour l'affichage d'un FastBitmap dans un canvas
//------------------------------------------------------------------------------
// Credits :
// Composant original :  FCL Compact Image LCL Interface (c) 2016, Paul F. Michell.
//------------------------------------------------------------------------------
//==============================================================================
Unit CustomFastBitmapHandler;
//==============================================================================

{$I beanz.inc}

//==============================================================================
Interface

//==============================================================================

Uses
  SysUtils, Graphics,GraphType, Types, FastBitmapCore;

//===[ TCUSTOMFASTBITMAPHANDLER ]===============================================
Type
  TCustomFastBitmapHandler = Class
  Private
    FFastBitmap: TFastBitmap;
    FImageBitmap: TBitmap;
    FRawImage: TRawImage;
    FColorFormat : TColorFormat;
    Function GetImageBitmap: TBitmap;
    procedure setFastBitmap(CompactImage: TFastBitmap);
    procedure setColorFormat(value:TColorFormat);
  Public
    Constructor Create(CompactImage: TFastBitmap);
    Destructor Destroy; Override;

    Function Description: String;
    Procedure Update;
    //Procedure DrawTo; virtual; abstract;

    Property ImageBitmap: TBitmap Read GetImageBitmap;
    Property RawImage: TRawImage Read FRawImage;
    property FastBitmap : TFastBitmap Read FFastBitmap write setFastBitmap;
    property ColorFormat : TColorFormat Read FColorFormat write setColorFormat default cfRGBA;
  End;

//==============================================================================

Implementation

//==============================================================================

//------------------------------------------------------------------------------
// Creation de TCustomFastBitmapHandler en liant un TFastBitmap
//------------------------------------------------------------------------------
Constructor TCustomFastBitmapHandler.Create(CompactImage: TFastBitmap);
Begin
  FFastBitmap := CompactImage;
  FImageBitmap := TBitmap.Create;
  setColorFormat(cfBGRA);
  Update;
End;

//------------------------------------------------------------------------------
// Destruction de TCustomFastBitmapHandler
//------------------------------------------------------------------------------
Destructor TCustomFastBitmapHandler.Destroy;
Begin
  FImageBitmap.Free;
  Inherited Destroy;
End;

//------------------------------------------------------------------------------
// Definit un nouveau TFastBitmap
//------------------------------------------------------------------------------
procedure TCustomFastBitmapHandler.setFastBitmap(CompactImage: TFastBitmap);
Begin
  FFastBitmap := CompactImage;
  Update;
End;

//------------------------------------------------------------------------------
// Renvoit des informations sur le "RawImage"
//------------------------------------------------------------------------------
Function TCustomFastBitmapHandler.Description: String;
Begin
  Result := StringReplace(FRawImage.Description.AsString, ' ', LineEnding, [rfReplaceAll]);
End;

//------------------------------------------------------------------------------
// Definit le mode de couleur pour l'affichage et la definition du "RawImage"
//------------------------------------------------------------------------------
Procedure TCustomFastBitmapHandler.setColorFormat(value:TColorFormat);
Begin
  if (FColorFormat = Value) then exit;
  Case value of
    cfBGRA :
      begin
        FRawImage.Description.Init_BPP32_B8G8R8A8_BIO_TTB(FFastBitmap.Width, FFastBitmap.Height);
      end;
    cfRGBA :
      begin
        FRawImage.Description.Init_BPP32_R8G8B8A8_BIO_TTB(FFastBitmap.Width, FFastBitmap.Height);
      end;
  end;
  FColorFormat := Value;
  Update;
End;

//------------------------------------------------------------------------------
// Mise à jour des donnée du "RawImage" par rapport au TFastBitmap
//------------------------------------------------------------------------------
Procedure TCustomFastBitmapHandler.Update;
begin
  //FRawImage.Height := FFastBitmap.Height;
  //FRawImage.Width := FFastBitmap.Width;
  Case FColorFormat of
    cfBGRA :
      begin
        FRawImage.Description.Init_BPP32_B8G8R8A8_BIO_TTB(FFastBitmap.Width, FFastBitmap.Height);
      end;
    cfRGBA :
      begin
        FRawImage.Description.Init_BPP32_R8G8B8A8_BIO_TTB(FFastBitmap.Width, FFastBitmap.Height);
      end;
  end;

  FRawImage.Data := PByte(FFastBitmap.Data);
  FRawImage.DataSize := FRawImage.Description.BytesPerLine*FFastBitmap.Height;
end;

//------------------------------------------------------------------------------
// Retourne un TBitmap
//------------------------------------------------------------------------------
Function TCustomFastBitmapHandler.GetImageBitmap: TBitmap;
Begin
  Update;
  FImageBitmap.LoadFromRawImage(FRawImage, false);
  Result := FImageBitmap;
End;

//==============================================================================

End.

