//==============================================================================
// FastImage.pas
//------------------------------------------------------------------------------
// Historique :
// 18/11/16 - BeanzMaster - Creation
//------------------------------------------------------------------------------
// Description :
// Composant visuel de remplacement à TImage utilisant les composants
// "TFastBitmap" et "TLCLFastImageHandler"
//------------------------------------------------------------------------------
// Credits :
//------------------------------------------------------------------------------
//==============================================================================
unit FastImage;
//==============================================================================

{$I beanz.inc}

//==============================================================================
interface

uses
  Classes, SysUtils, types, LResources, Forms, Controls, Graphics, Dialogs,
  fastbitmapcore, lclfastbitmaphandler;

//===[ TFASTCUSTOMIMAGE ]=======================================================
// Composant ancêtre pour l'affichage de bitmap utilisant TFastBitmap et
// TLCLFastBitmapHandler (TCustomFastBitmapHandler)
// Le code est directement tiré du code source de TImage de la LCL
//------------------------------------------------------------------------------
type
  TCustomFastImage = class(TGraphicControl)
  private
    { Private declarations }

    FLCLBitmapHandler : TLCLFastBitmapHandler;
    FFastBitmap : TFastBitmap;
    //FAntialiasingMode: TAntialiasingMode;

    FOnPictureChanged: TNotifyEvent;

    FCenter: Boolean;
    FProportional: Boolean;
    //FTransparent: Boolean;
    FStretch: Boolean;

    FPainting: boolean;

    function  GetCanvas: TCanvas;
    procedure SetPicture(const AValue: TFastBitmap);
    procedure SetCenter(const AValue : Boolean);
    procedure SetProportional(const AValue: Boolean);
    procedure SetStretch(const AValue : Boolean);
  protected
    { Protected declarations }
    procedure PictureChanged(Sender : TObject); virtual;
    function ComputeDestRect: TRect;
    procedure CalculatePreferredSize(var PreferredWidth,
                                         PreferredHeight: integer;
                                         WithThemeSpace: Boolean); override;
    class function GetControlClassDefaultSize: TSize; override;
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Invalidate; override;

    property Canvas: TCanvas read GetCanvas;
  published
    { Published declarations }
    property Align;
    property AutoSize;
    property Center: Boolean read FCenter write SetCenter default False;

    property Constraints;
    property Picture: TFastBitmap read FFastBitmap write setPicture;
    property Visible;
    property OnClick;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property Stretch: Boolean read FStretch write SetStretch default False;
    //property Transparent: Boolean read FTransparent write SetTransparent default False;
    property Proportional: Boolean read FProportional write SetProportional default False;
    property OnPictureChanged: TNotifyEvent read FOnPictureChanged write FOnPictureChanged;
  end;

//===[ TFASTIMAGE ]=============================================================
Type
  TFastImage = Class(TCustomFastImage)
    published
    //property AntialiasingMode;
    property Align;
    property Anchors;
    property AutoSize;
    property BorderSpacing;
    property Center;
    property Stretch;
    property Proportional;
    property Constraints;
    property DragCursor;
    property DragMode;
    property Enabled;
    property OnChangeBounds;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnPaint;
    //property OnPictureChanged;
    property OnResize;
    property OnStartDrag;
    property ParentShowHint;
    property Picture;
    property PopupMenu;
    property Hint;
    property ShowHint;
    //property Transparent;
    property Visible;
  end;

//==============================================================================

procedure Register;

//==============================================================================
implementation

//==============================================================================

//------------------------------------------------------------------------------
// Enregistrement du composant TFastImage dans la palette de composants "Beanz Controls"
//------------------------------------------------------------------------------
procedure Register;
begin
  {$I fastimage_icon.lrs}
  RegisterComponents('Beanz Controls',[TFastImage]);
end;

//===[ TFASTCUSTOMIMAGE ]=======================================================

//------------------------------------------------------------------------------
// Creation de TFastImage
//------------------------------------------------------------------------------
constructor TCustomFastImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle:= [csCaptureMouse, csClickEvents, csDoubleClicks];
  AutoSize := False;
  FCenter := False;
  FProportional := False;
  FStretch := False;
//  FTransparent := False;
  with GetControlClassDefaultSize do
  begin
    SetInitialBounds(0, 0, CX, CY);
    FFastBitmap := TFastBitmap.Create(CX,CY);
    FFastBitmap.OnChange := @PictureChanged;
  end;
  FLCLBitmapHandler := TLCLFastBitmapHandler.Create(FFastBitmap);
end;

//------------------------------------------------------------------------------
// Destruction de TFastImage
//------------------------------------------------------------------------------
destructor TCustomFastImage.Destroy;
begin
 // FPicture.OnChange := nil;
 // FPicture.Graphic := nil;
  FreeAndNil(FFastBitmap);
  FreeAndNil(FLCLBitmapHandler);
  inherited Destroy;
end;

//------------------------------------------------------------------------------
// Retourne le "canvas" de TGraphiControl
//------------------------------------------------------------------------------
function TCustomFastImage.GetCanvas: TCanvas;
begin
    Result := inherited Canvas;
end;

{ procedure TCustomImage.SetAntialiasingMode(AValue: TAntialiasingMode);
begin
  if FAntialiasingMode = AValue then Exit;
  FAntialiasingMode := AValue;
  PictureChanged(Self);
end; }

//------------------------------------------------------------------------------
// Assigne un bitmap de type TFastBitmap
//------------------------------------------------------------------------------
procedure TCustomFastImage.SetPicture(const AValue: TFastBitmap);
begin
  if FFastBitmap=AValue then exit;
  //the OnChange of the picture gets called and
  // notifies this TCustomImage that something changed.
  //FPicture.Assign(AValue);
  FFastBitmap.Assign(AValue);
  FFastBitmap.OnChange := @PictureChanged;
  FLCLBitmapHandler.Update;

end;

//------------------------------------------------------------------------------
// Definit si on doit recadrer le bitmap en fonction de la dimension de TFastImage
//------------------------------------------------------------------------------
procedure TCustomFastImage.SetStretch(const AValue : Boolean);
begin
  if FStretch = AValue then exit;
  FStretch := AValue;
  PictureChanged(Self);
end;

{procedure TCustomFastImage.SetTransparent(const AValue : Boolean);
begin
  if FTransparent = AValue then exit;
  FTransparent := AValue;
  if (FPicture.Graphic <> nil) and (FPicture.Graphic.Transparent <> FTransparent)
  then FPicture.Graphic.Transparent := FTransparent
  else PictureChanged(Self);
end;
}
//------------------------------------------------------------------------------
// Definit si on doit centrer le bitmap dans TFastImage
//------------------------------------------------------------------------------
 procedure TCustomFastImage.SetCenter(const AValue : Boolean);
begin
  if FCenter = AValue then exit;
  FCenter := AValue;
  PictureChanged(Self);
end;

//------------------------------------------------------------------------------
// Definit si l'on doit conserver une taille proprotionnel du bitmap lors du
// redimensionnement
//------------------------------------------------------------------------------
procedure TCustomFastImage.SetProportional(const AValue: Boolean);
begin
  if FProportional = AValue then exit;
  FProportional := AValue;
  PictureChanged(Self);
end;

//------------------------------------------------------------------------------
// Notification de changment de dimension du TFastImage, et rafraichissement de celui-ci
//------------------------------------------------------------------------------
procedure TCustomFastImage.PictureChanged(Sender : TObject);
begin
    if AutoSize then
    begin
      InvalidatePreferredSize;
      AdjustSize;
    end;
   // On appel notre Handler (TLCLFastBitmapHandler).
   // On met a jour les données de notre TFastBitmap
   FLCLBitmapHandler.Update;

  invalidate;
  if Assigned(OnPictureChanged) then OnPictureChanged(Self);
end;

//------------------------------------------------------------------------------
// Calcul d'un TRect pour le positionnement et la dimension du Bitmap en fonction
// des propriété "Stretch, Proportional et Center" lors de l'affichage dans TFastImage
//------------------------------------------------------------------------------
function TCustomFastImage.ComputeDestRect: TRect;
var
  PicWidth: Integer;
  PicHeight: Integer;
  ImgWidth: Integer;
  ImgHeight: Integer;
  w: Integer;
  h: Integer;
begin
  PicWidth := Picture.Width;
  PicHeight := Picture.Height;
  ImgWidth := ClientWidth;
  ImgHeight := ClientHeight;
  if Stretch or (Proportional and ((PicWidth > ImgWidth) or (PicHeight > ImgHeight))) then
  begin
    if Proportional and (PicWidth > 0) and (PicHeight > 0) then
    begin
      w:=ImgWidth;
      h:=(PicHeight*w) div PicWidth;
      if (h>ImgHeight) then
      begin
        h:=ImgHeight;
        w:=(PicWidth*h) div PicHeight;
      end;
      PicWidth:=w;
      PicHeight:=h;
    end
    else begin
      PicWidth := ImgWidth;
      PicHeight := ImgHeight;
    end;
  end;

  Result:=Rect(0,0,PicWidth,PicHeight);

  if Center then
    OffsetRect(Result,(ImgWidth-PicWidth) div 2,(ImgHeight-PicHeight) div 2);
end;

//------------------------------------------------------------------------------
// Invalidate
//------------------------------------------------------------------------------
procedure TCustomFastImage.Invalidate;
begin
  if FPainting then exit;
  inherited Invalidate;
end;

//------------------------------------------------------------------------------
// Calcul dimension "préférée"
//------------------------------------------------------------------------------
procedure TCustomFastImage.CalculatePreferredSize(var PreferredWidth,
  PreferredHeight: integer; WithThemeSpace: Boolean);
begin
  PreferredWidth := Picture.Width;
  PreferredHeight := Picture.Height;
end;

//------------------------------------------------------------------------------
// Renvoie la dimension par défault du TFastBitmap
//------------------------------------------------------------------------------
class function TCustomFastImage.GetControlClassDefaultSize: TSize;
begin
  Result.CX := 90;
  Result.CY := 90;
end;

//------------------------------------------------------------------------------
// Affichage de notre composant
//------------------------------------------------------------------------------
procedure TCustomFastImage.Paint;

  procedure DrawFrame;
  begin
    with inherited Canvas do
    begin
      Pen.Color := clBlack;
      Pen.Style := psDash;
      MoveTo(0, 0);
      LineTo(Self.Width-1, 0);
      LineTo(Self.Width-1, Self.Height-1);
      LineTo(0, Self.Height-1);
      LineTo(0, 0);
    end;
  end;

var
  R: TRect;
  C: TCanvas;
begin

  if csDesigning in ComponentState then DrawFrame;

  C := inherited Canvas;
  R := ComputeDestRect;

  FPainting:=true;
  try
    // On appel notre Handler (TLCLFastBitmapHandler)
    // On Affiche notre TFastBitmap
    FLCLBitmapHandler.DrawTo(C,R);
  finally
    FPainting:=false;
  end;


end;

//==============================================================================

end.
