//==============================================================================
// LCLFastBitmapHandler.pas
//------------------------------------------------------------------------------
// Historique :
// 18/11/16 - BeanzMaster - Creation
//------------------------------------------------------------------------------
// Description :
// Composant graphique optimisé pour l'affichage d'un FastBitmap dans un canvas (LCL)
//
// Credits :
// Composant original :  FCL Compact Image LCL Interface (c) 2016, Paul F. Michell.
//------------------------------------------------------------------------------
//==============================================================================
Unit LCLFastBitmapHandler;
//==============================================================================

{$I Beanz.inc}

//==============================================================================
Interface

//==============================================================================

Uses
  SysUtils,
  {$IFDEF WINDOWS}Windows,{$ENDIF}
  {$IFDEF LCLGTK2}GDK2, Gtk2Def,{$ENDIF}
  {$IFDEF LCLQT}QT4, QTWidgets, QTObjects,{$ENDIF}
  {$IFDEF LCLCARBON}MacOSAll, CarbonCanvas,{$ENDIF}
  Graphics, GraphType, CustomFastBitmapHandler;


//===[ TLCLFASTBITMAPHANDLER ]==================================================
Type
  TLCLFastBitmapHandler = Class(TCustomFastBitmapHandler)
  Public
    Procedure DrawTo(Canvas: TCanvas;R:TRect);
  End;

//==============================================================================
Implementation

//------------------------------------------------------------------------------
// Affiche un TFastBitmap sur un "Canvas"
//------------------------------------------------------------------------------
Procedure TLCLFastBitmapHandler.DrawTo(Canvas: TCanvas;R:TRect);
{$IFDEF WINDOWS}
Var
  BitsInfo: TBitmapInfo;
Begin
  With BitsInfo.bmiHeader Do
    Begin
      biSize := SizeOf(TBitmapInfoHeader);
      biWidth := FastBitmap.Width;
      biHeight := -FastBitmap.Height; { Note: negative height flags image as bottom to top scanline order. }
      biPlanes := 1;
      biBitCount := RawImage.Description.BitsPerPixel;
      biCompression := BI_RGB;
      biSizeImage :=  RawImage.DataSize;
      biXPelsPerMeter := 0;
      biYPelsPerMeter := 0;
      biClrUsed := 0;
      biClrImportant := 0;
    End;
  Canvas.Lock;
  StretchDIBits(Canvas.Handle, 0, 0, FastBitmap.Width, FastBitmap.Height,
                 R.Left, R.Top, (R.Right-R.Left), (R.Bottom-R.Top), RawImage.Data,
                //0, 0, FastBitmap.Width, FastBitmap.Height, RawImage.Data,
                BitsInfo, DIB_RGB_COLORS, SRCCOPY);
  Canvas.Unlock;
End;
{$ELSE}{$IFDEF LCLGTK2}
Begin
  gdk_draw_rgb_32_image(TGTKDeviceContext(Canvas.Handle).Drawable,
                        TGTKDeviceContext(Canvas.Handle).GC, 0, 0,
                        FCompactImage.Width, FCompactImage.Height,
                        GDK_RGB_DITHER_NORMAL,
                        FRawImage.Data, FRawImage.Description.BytesPerLine);
End;

{$ELSE}{$IFDEF LCLQT}
Var
  QtImage: TQtImage;
  QtContext: TQtDeviceContext;
  Rect: TRect;
Begin
  QtImage := TQtImage.Create(FRawImage.Data, FCompactImage.Width, FCompactImage.Height, QImageFormat_ARGB32);
  Rect := R; //Types.Rect(0, 0, FCompactImage.Width, FCompactImage.Height);
  QtContext := TQtDeviceContext(Canvas.Handle);
  QtContext.drawImage(@Rect, QtImage.Handle, @Rect, Nil, @Rect);
  QtImage.Free;
End;
{$ELSE}{$IFDEF LCLCARBON}
Var
  Image: CGImageRef;
  BitmapReference: CGContextRef;
Begin
  BitmapReference := CGBitmapContextCreate(FRawImage.Data, FCompactImage.Width, FCompactImage.Height,
                                           8, 4*FCompactImage.Width,
                                           CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB),
                                           kCGImageAlphaPremultipliedLast);
  Image := CGBitmapContextCreateImage(BitmapReference);
 // TCarbonDeviceContext(Canvas.Handle).DrawCGImage(0, 0, FCompactImage.Width, FCompactImage.Height, Image);
  TCarbonDeviceContext(Canvas.Handle).DrawCGImage(R.Left, R.Top, (R.Right-R.Left), (R.Bottom-R.Top), Image);
  CGImageRelease(Image);
  CGContextRelease(BitmapReference);
End;
{$ELSE}
Begin
  GetImageBitmap;
  Canvas.Lock;
  //Canvas.Draw(0,0, ImageBitmap);
  Canvas.Draw(R.Left, R.Top, ImageBitmap);
  Canvas.Unlock;
End;
{$ENDIF}{$ENDIF}{$ENDIF}{$ENDIF}


End.

